
class 'App'

App.MENU_PATH = "Instrument Box:Delete Used Instruments"

function App:__init()

  -- add menu entry to Renoise interface
  renoise.tool():add_menu_entry {
    name = App.MENU_PATH,
    invoke = function() self:delete_used_instruments() end
  }

end


function App:delete_used_instruments()

  -- localize/'cache' frequently used objects
  local song = renoise.song()
  local pat_iter = song.pattern_iterator
  
  -- make instrument table
  local instruments = table.create()
  for inst_idx, state in ipairs(song.instruments) do
    instruments[inst_idx] = false
  end

  -- scan patterns and set all used instruments to 'true' in instrument table
  for pat_idx, pat in ipairs(song.patterns) do
    for track_idx, track in ipairs(pat.tracks) do
      if song:track(track_idx).type == renoise.Track.TRACK_TYPE_SEQUENCER and
        (not track.is_empty) then
        local visible_note_columns = song:track(track_idx).visible_note_columns
        for line_idx, line in ipairs(track.lines) do
          if not line.is_empty then
            for ncol_idx = 1, visible_note_columns do
              local ncol = line:note_column(ncol_idx)
              if not ncol.is_empty then
                instruments[ncol.instrument_value+1] = true
              end
            end
          end
        end
      end
    end
  end
  
  -- special case for notes with 'empty' instrument value (255).
  instruments[255+1] = nil

  -- special case: if all instruments are used, insert an empty instrument
  local used_amt = 0
  for inst_idx, is_used in ripairs(instruments) do
    used_amt = used_amt + (is_used and 1 or 0)
  end
  if used_amt == #instruments then
    song:insert_instrument_at(#song.instruments+1)
  end

  -- delete used instruments
  for inst_idx, is_used in ripairs(instruments) do
    if is_used then
      song:delete_instrument_at(inst_idx)
    end
  end

  renoise.app():show_status(
    ("Deleted %d used instruments."):format(used_amt)
  )
 
end

App()

